function [w,b,ind,xind] = AL1MSVM(xtrain,ytrain,lambda,weight) 
% Adaptive L1 MSVM
%Input:
%   xtrain : n-by-d data matrix to train
%   ytrain : column vector of target {1, 2,...,k}'s
%   lambda : tuning parameter in scad function (default : 2 or whatever the paper uses)
% Output:
%   w : direction vector of separating hyperplane
%   b : the bias
%   Ind : Indices of remained variables
if nargin < 3;
    disp('not enough input');
elseif nargin == 3;
    weight = 1;
end;

global Result Prob
K = max(ytrain); %the number of classes
n = size(xtrain,1);
d = size(xtrain,2); 
w = zeros(d*K,1);
b = zeros(K,1);
xi = ones(n*K,1);
eta = zeros(d*K,1);
xind = 1:d;
A = ones(n,1)*(1:K);
ytrain2 = ytrain*ones(1,K);
A = (A~=ytrain2);
A = A(:);
Ab = [ones(n,1) zeros(n,K-1)];
for i = 1:(K-1)
    Ab = [Ab;zeros(n,i) ones(n,1) zeros(n,K-1-i)];
end
Aw1 = [xtrain zeros(n,d*(K-1))];
for i = 1:(K-1)
    Aw1 = [Aw1;zeros(n,i*d) xtrain zeros(n,d*(K-1-i))];
end
Axi = diag(-ones(1,n*K));
Aineq = [Ab Aw1 Axi zeros(n*K,d*K)];

aww = [zeros(K*d,K) eye(K*d) zeros(K*d,n*K) -eye(K*d);
       zeros(K*d,K) -eye(K*d) zeros(K*d,n*K) -eye(K*d)];
Aineq = [Aineq;aww];
bineq = [-ones(n*K,1);zeros(2*K*d,1)];    

aew = eye(d);
for i = 1:(K-1)
    aew = [aew eye(d)];
end    
Aeq = [zeros(d,K) aew zeros(d,n*K+d*K)];    
Aeq = [ones(1,K) zeros(1,K*d+n*K+d*K);Aeq];
beq = zeros(1+d,1);
lb = [-inf*ones(K*(1+d),1);zeros(n*K+d*K,1)];
ub = inf*ones(K*(1+d)+n*K+K*d,1);

f = [zeros(K,1);zeros(K*d,1);A/n;repmat(lambda,K*d,1).*weight];
Prob = lpAssign(f,[Aineq;Aeq],[-inf*ones(K*(n+2*d),1);beq],[bineq;beq],lb,ub,[],'NL');
Prob = ProbCheck(Prob, 'cplex');
Result = cplexTL(Prob);
bwxe=Result.x_k;
b = bwxe(1:K);
w = bwxe(K+1:K+K*d);
% xi = bwxe(K+K*d+1:K+K*d+n*K);
eta = bwxe(K+K*d+n*K+1:end);
ind = eta>1e-4;
ind2 = reshape(ind,d,K);
ind2 = ind2';
ind3 = (sum(ind2))>=1;
xind = xind(ind3);
w = reshape(w,d,K);
w = w';
    
