function [w,b,xind,ind] = LRcompositeMCPLLA(xtrain,ytrain,lambda) 
% Use LLA algorithm to implement composite MCP LR classification 
% Input:
%   xtrain : n-by-d data matrix to train
%   ytrain : column vector of target {1, 2,...,k}'s
%   lambda : tuning parameter in scad function
% Output:
%   w : direction vector of separating hyperplane
%   b : the bias
%   Ind : Indices of remained variables

maxit = 100;
iter = 0;
tol = 1e-6;
a=3;  %tuning parameter in MCP function, could be3 or 30

global Result Prob
K = max(ytrain); %the number of classes
n = size(xtrain,1);
d = size(xtrain,2);

bb = 0.5*a*K*lambda;

SB = mnrfit(xtrain,ytrain);
SB = [SB zeros(d+1,1)];
B = SB-repmat(sum(SB,2)/K,1,K);

% B = zeros(d+1,K);
b = B(1,:)';
w = B(2:(d+1),:);
w = w(:);

% % eta_kj=|w_kj|
eta = abs(w);
oldbw = B(:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

bindex = 1:(d+1):1+(K-1)*(d+1);
windex = 2:(d+1);
for i = 1:(K-1)
    wi = (2+(d+1)*i):((d+1)+(d+1)*i);
    windex = [windex wi];
end  

diff1 = 100;
diff2 = 100;
xind = 1:d;
ind = zeros(d,1);

xtrain2 = [ones(n,1) xtrain];
ytrain2 = ( ytrain*ones(1,K) ) == ( ones(n,1)*(1:K) ); 

a1 = eye(d);
a1 = [zeros(d,1) a1];
a2 = [a1 zeros(d, (K-1)*(d+1))];
for i = 1:(K-1)
    a3 = [zeros(d,(d+1)*i) a1 zeros(d,(d+1)*(K-1-i))];
    a2 = [a2;a3];
end
a4 = -eye(d*K);
% % w_kj-eta_kj<=0, -w_kj-eta_kj<=0
Aineq = [a2 a4;-a2 a4];
bineqU = zeros(2*K*d,1);
bineqL = -inf*ones(2*K*d,1);

a5 = eye(d+1);
a5 = repmat(a5,1,K);
% % w_1j+w_2j+...+w_Kj=0, for j=0,1,2,...d
Aeq = [a5 zeros(d+1,d*K)];
beq = zeros(d+1,1);
Aineq = [Aineq;Aeq];
bineqU = [bineqU;beq];
bineqL = [bineqL;beq];

[P,lnL] = lik(xtrain2,ytrain2,B,K,n);
[g,H] = dev(xtrain2,ytrain2,P,d,K,n);
oldobj = lnL;
                                          
while (diff1 > tol) & (iter < maxit)
    
 iter = iter + 1;

 F = [H/n zeros(K*(d+1),d*K);zeros(d*K,K*(d+1)+d*K)];
 
 p = MCP(a,lambda,eta);
 dp = MCPdev(a,lambda,eta);
 sz = [d K];
 pm = reshape(p,sz);
 pm2 = sum(pm,2);
 dpm2 = MCPdev(bb,lambda,pm2);
 dpm3 = repmat(dpm2,K,1);
 dp2 = dpm3.*dp;                                                                                                            
 
 f1 = g(:)/n -(H/n)*B(:); 
 f = [f1;dp2];
 
 
 % % eta_kj>=0, for j=1,,,,d
x_L = -inf((d+1)*K,1);
x_L = [x_L;zeros(d*K,1)];
x_U = inf((d+1)*K+d*K,1);


Prob = qpAssign(sparse(F),f,Aineq,bineqL,bineqU,x_L,x_U,[],'NL');
Prob = ProbCheck(Prob, 'sqopt');
    while (diff2 > tol)
      Result = sqoptTL(Prob);
      bwe = Result.x_k;
      bw = bwe(1:(d+1)*K);
      b = bw(bindex);
      nw = bw(windex);
      eta = bwe(((d+1)*K+1):end);
      diff2 = norm(bw-oldbw);
      xind = 1:d;
      oldbw = bw;
      
      eta2 = reshape(eta,sz);
      eta3 = max(eta2,[],2);
      ind = eta3>1e-4;
      xind = xind(ind);
      
      p = MCP(a,lambda,eta);
      dp = MCPdev(a,lambda,eta);
      pm = reshape(p,sz);
      pm2 = sum(pm,2);
      dpm2 = MCPdev(bb,lambda,pm2);
      dpm3 = repmat(dpm2,K,1);
      dp2 = dpm3.*dp;

      f = [f1;dp2];   
      Prob.QP.c = f;
      Prob = ProbCheck(Prob, 'sqopt');
    end 
    
  w = reshape(nw,d,K);
  B = [b';w];
 
  [P,lnL] = lik(xtrain2,ytrain2,B,K,n);
  [g,H] = dev(xtrain2,ytrain2,P,d,K,n);
  
  p = MCP(a,lambda,eta);
  pm = reshape(p,sz);
  pm2 = sum(pm,2);
  pm3 = MCP(bb,lambda,pm2);
  obj = lnL+n*sum(pm3);
  
  diff1 = abs(oldobj - obj);
  oldobj = obj;
end
  

  function [P,lnL] = lik(xtrain2,ytrain2,B,K,n)
% PURPOSE: Computes value of negative log likelihood function for multinomial logit regression

xb = xtrain2*B;
e_xb = exp(xb);
sum_e_xb = sum(e_xb,2);
P = e_xb./repmat(sum_e_xb,1,K);

lnp = log(P);
contribution = ytrain2 .* lnp;
lnL = - sum(sum(contribution))/n;



  
function [g,H] = dev(xtrain2,ytrain2,P,d,K,n)
% PURPOSE: Computes gradient and Hessian of the negative log-likelihood of
% the symmetric multinomial logit model

tmp = P - ytrain2;
g = xtrain2'*tmp;

H = zeros(K*(d+1));
for m = 1:K 
    for l = 1:K
        fr = (m-1)*(d+1) + 1;
        lr = m*(d+1);
        fc = (l-1)*(d+1) + 1;
        lc = l*(d+1);       
        index = (l==m);
        index = repmat(index,n,1);
        H(fr:lr,fc:lc) = ((xtrain2.*(P(:,m)*ones(1,(d+1))))'*(xtrain2.*((index-P(:,l))*ones(1,(d+1)))));
    end;
end;

  