function [w,b,xind,ind] = LRsupSCADLLA(xtrain,ytrain,lambda,B) 
% Use LLA algorithm to implement supSCAD LR classification 
% Input:
%   xtrain : n-by-d data matrix to train
%   ytrain : column vector of target {1, 2,...,k}'s
%   lambda : tuning parameter in scad function
%   B: ridge estimator as the starting point
% Output:
%   w : direction vector of separating hyperplane
%   b : the bias
%   Ind : Indices of remained variables


maxit = 100;
iter1 = 0;
iter2 = 0;
tol = 1e-6;
a=3.7;  %tuning parameter in scad function
global Result Prob
K = max(ytrain); %the number of classes
n = size(xtrain,1);
d = size(xtrain,2);


% SB = mnrfit(xtrain,ytrain);
% SB = [SB zeros(d+1,1)];
% B = SB-repmat(sum(SB,2)/K,1,K);
b = B(1,:)';
w = B(2:end,:);
w = w(:);
oldbw = B(:);
eta = max(abs(B(2:end,:)),[],2);
xind = 1:d;
ind = ones(d,1);
 

% % b_1,w_11,,,,w_1d,b_2,w_21,,,,w_2d,,,,,,,b_k,w_k1,,,,w_kd
bindex = 1:(d+1):1+(K-1)*(d+1);
windex = 2:(d+1);
for i = 1:(K-1)
    wi = (2+(d+1)*i):((d+1)+(d+1)*i);
    windex = [windex wi];
end  

diff1 = 100;
diff2 = 100;

xtrain2 = [ones(n,1) xtrain];
ytrain2 = (ytrain*ones(1,K))==(ones(n,1)*(1:K)); 

a1 = [zeros(d,1) eye(d)];
a2 = zeros(d,d+1);
a3 = [a1 repmat(a2,1,K-1)];

for i = 1:(K-1)
    a4 = [repmat(a2,1,i) a1 repmat(a2,1,K-1-i)];
    a3 = [a3;a4];
end

a5 = -eye(d);
a5 = repmat(a5,K,1);

% % beta_kj-eta_j<=0, -beta_kj-eta_j<=0
Aineq = [a3 a5;-a3 a5];
bineqU = zeros(2*K*d,1);
bineqL = -inf*ones(2*K*d,1);

a6 = eye(1+d);
% % b_1+b_2+,,,+b_K=0;
% % beta_1j+,,,,+beta_Kj=0, for j=1,2,,,,d 
Aeq = [repmat(a6,1,K) zeros(1+d,d)];
beq = zeros(d+1,1);
Aineq = [Aineq;Aeq];
bineqU = [bineqU;beq];
bineqL = [bineqL;beq];


% % eta_j>=0, for j=1,,,,d
x_L = -inf((d+1)*K,1);
x_L = [x_L;zeros(d,1)];
x_U = inf((d+1)*K+d,1);


[P,prelnL] = lik(xtrain2,ytrain2,B,K,n);
[g,H] = dev(xtrain2,ytrain2,P,d,K,n);

while (iter1 < maxit) & (diff1 > tol)
 iter1 = iter1 + 1;

 F = [H/n zeros(K*(d+1),d);zeros(d,K*(d+1)+d)];

 dp = scaddp(a,lambda,eta);
 f1 = g(:)/n -(H/n)*B(:);
 f = [f1;dp];

Prob = qpAssign(sparse(F),f,Aineq,bineqL,bineqU,x_L,x_U,[],'NL');
Prob = ProbCheck(Prob, 'sqopt');
    while (diff2>tol) & (iter2<1000)
      iter2=iter2+1;
      Result = sqoptTL(Prob);
      bwe = Result.x_k;
      bw = bwe(1:(d+1)*K);
      nb = bw(bindex);
      nw = bw(windex);
      eta = bwe(((d+1)*K+1):end);
      diff2 = norm(bw-oldbw);
      nxind = 1:d;
      nind = eta>1e-4;
      nxind = nxind(nind);
      oldbw = bw;
     dp = scaddp(a,lambda,eta);
     f = [f1;dp];
      Prob.QP.c=f;
      Prob = ProbCheck(Prob, 'sqopt');
    end 
  ww = reshape(nw,d,K);
  nB = [nb';ww];
 [P,postlnL] = lik(xtrain2,ytrain2,nB,K,n);
 [g,H] = dev(xtrain2,ytrain2,P,d,K,n);
  
 
diff1 = norm(B(:)-nB(:));
    B = nB;
    
    b = B(1,:)';
    
    
    w1 = B(2:(d+1),:);
    w2 = w1';
    ind=nind;
    xind=nxind;
    id = ones(K,1)*ind';
    w = w2.*id;
    
end

   
  
  function [P,nlnL] = lik(xtrain2,ytrain2,B,K,n)
% PURPOSE: Computes value of negative log likelihood function for multinomial logit regression
xb = xtrain2*B;
e_xb = exp(xb);
sum_e_xb = sum(e_xb,2);
P = e_xb./repmat(sum_e_xb,1,K);

lnp = log(P);
contribution = ytrain2 .* lnp;
nlnL = - sum(sum(contribution))/n;



function [g,H] = dev(xtrain2,ytrain2,P,d,K,n)
% PURPOSE: Computes gradient and Hessian of the negative log-likelihood of
% the symmetric multinomial logit model
tmp = P - ytrain2;
g = xtrain2'*tmp;

H = zeros(K*(d+1));
for m = 1:K
    for l = 1:K
        fr = (m-1)*(d+1) + 1;
        lr = m*(d+1);
        fc = (l-1)*(d+1) + 1;
        lc = l*(d+1);       
        index = (l==m);
        index = repmat(index,n,1);
        H(fr:lr,fc:lc) = ((xtrain2.*(P(:,m)*ones(1,(d+1))))'*(xtrain2.*((index-P(:,l))*ones(1,(d+1)))));
    end;
end;
